=== modified file 'NEWS'
--- NEWS	2011-02-01 17:18:37 +0000
+++ NEWS	2011-02-18 11:53:18 +0000
@@ -14,6 +14,10 @@
 
   * Mark as compatible with Bazaar 2.4. (Jelmer Vernooij)
 
+ FEATURES
+ 
+  * Add lock dialog. (Daniel Schierbeck, #93652)
+
 0.99.1	2010-08-26
 
  BUG FIXES

=== modified file 'branchview/treeview.py'
--- branchview/treeview.py	2009-06-15 14:02:08 +0000
+++ branchview/treeview.py	2011-02-18 11:53:18 +0000
@@ -21,6 +21,7 @@
 from graphcell import CellRendererGraph
 from treemodel import TreeModel
 from bzrlib.revision import NULL_REVISION
+from bzrlib.plugins.gtk import lock
 
 
 class TreeView(gtk.VBox):
@@ -225,19 +226,16 @@
     def add_tag(self, tag, revid=None):
         if revid is None: revid = self.revision.revision_id
 
-        try:
-            self.branch.unlock()
-
+        if lock.release(self.branch):
             try:
-                self.branch.lock_write()
+                lock.acquire(self.branch, lock.WRITE)
                 self.model.add_tag(tag, revid)
             finally:
-                self.branch.unlock()
-
-        finally:
-            self.branch.lock_read()
-
-        self.emit('tag-added', tag, revid)
+                lock.release(self.branch)
+
+            lock.acquire(self.branch, lock.READ)
+
+            self.emit('tag-added', tag, revid)
         
     def refresh(self):
         gobject.idle_add(self.populate, self.get_revision())

=== added file 'lock.py'
--- lock.py	1970-01-01 00:00:00 +0000
+++ lock.py	2011-02-18 11:53:18 +0000
@@ -0,0 +1,62 @@
+
+import gtk
+
+
+RESPONSE_BREAK = 0
+RESPONSE_CANCEL = gtk.RESPONSE_CANCEL
+
+READ = "read"
+WRITE = "write"
+
+
+def acquire(branch, lock_type):
+    if branch.get_physical_lock_status():
+        dialog = LockDialog(branch)
+        response = dialog.run()
+        dialog.destroy()
+
+        if response == RESPONSE_BREAK:
+            branch.break_lock()
+        else:
+            return False
+
+    if lock_type == READ:
+        branch.lock_read()
+    elif lock_type == WRITE:
+        branch.lock_write()
+
+    return True
+
+
+def release(branch):
+    if branch.get_physical_lock_status():
+        dialog = LockDialog(branch)
+        response = dialog.run()
+        dialog.destroy()
+
+        if response == RESPONSE_BREAK:
+            branch.break_lock()
+        elif response == RESPONSE_CANCEL:
+            return False
+
+    branch.unlock()
+
+    return True
+
+
+class LockDialog(gtk.Dialog):
+
+    def __init__(self, branch):
+        gtk.Dialog.__init__(self)
+
+        self.branch = branch
+
+        self.set_title('Lock Not Held')
+
+        self.vbox.add(gtk.Label('This operation cannot be completed as ' \
+                                'another application has locked the branch.'))
+
+        self.add_button('Break Lock', RESPONSE_BREAK)
+        self.add_button(gtk.STOCK_CANCEL, RESPONSE_CANCEL)
+
+        self.vbox.show_all()

